{% extends 'admin/change_list.html' %}
{% load i18n %}

{% block breadcrumbs %}
    <div class="breadcrumbs">
        <a href="{% url 'admin:index' %}">{% translate 'Home' %}</a>
        &rsaquo; <a href="{% url 'admin:app_list' app_label=cl.opts.app_label %}">{{ cl.opts.app_config.verbose_name }}</a>
        &rsaquo; <a href="{% url 'admin:memberaudit_skillset_changelist' %}">{{ cl.opts.verbose_name_plural|capfirst }}</a>
        &rsaquo; {{ title }}
    </div>
{% endblock %}

{% block content %}
    <form action="" method="post">
        {% csrf_token %}
        {{ form.as_p }}
        <br>
        <br>
        <input type="submit" value="Submit">
        <a href="{% url 'admin:memberaudit_skillset_changelist' %}">
            <input type="button" name="Cancel" value="{% translate 'Cancel' %}">
        </a>
    </form>
{% endblock %}
