{% load i18n %}

<div role="tabpanel" class="tab-pane" id="corporation_compliance">
    <div class="table-responsive">
        <table class="table table-striped table-width-fix" id="tab_corporation_compliance">
            <thead>
                <tr>
                    <th>{% translate 'Organization' %}</th>
                    <th>{% translate 'Total Users' %}</th>
                    <th>{% translate 'Total Characters' %}</th>
                    <th>{% translate 'Compliance %' %}</th>
                </tr>
            </thead>

            <tbody></tbody>
        </table>
    </div>
    {% include 'memberaudit/partials/reports/tabs/_compliance_color_code.html' %}
</div>
