{% load i18n %}

<div role="tabpanel" class="tab-pane" id="mining_ledger">
    <h3>{% translate 'Mining' %}</h3>
    <div class="table-responsive">
        <table class="table table-striped table-width-fix" id="tab_mining_ledger">
            <thead>
                <tr>
                    <th>{% translate 'Date' %}</th>
                    <th>{% translate 'Type' %}</th>
                    <th>{% translate 'Quantity' %}</th>
                    <th>{% translate 'Price' %}</th>
                    <th>{% translate 'Solar System' %}</th>
                </tr>
            </thead>
            <tbody></tbody>
        </table>
    </div>
    {% include 'memberaudit/partials/character_viewer/tabs/update_status.html' with update_section=sections_update_status.mining_ledger %}
</div>
