# Generated by Django 4.0.10 on 2024-01-17 22:13

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("memberaudit", "0018_charactercloneinfo"),
    ]

    operations = [
        migrations.AlterField(
            model_name="charactercontractitem",
            name="record_id",
            field=models.PositiveBigIntegerField(db_index=True),
        ),
    ]
