# Generated by Django 4.0.10 on 2023-12-08 23:19

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("memberaudit", "0015_charactership_item_id"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="characterupdatestatus",
            name="parent_task_id",
        ),
        migrations.RemoveField(
            model_name="characterupdatestatus",
            name="root_task_id",
        ),
        migrations.AddField(
            model_name="characterupdatestatus",
            name="update_finished_at",
            field=models.DateTimeField(
                db_index=True,
                default=None,
                help_text="Finish time of an actual update",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="characterupdatestatus",
            name="update_started_at",
            field=models.DateTimeField(
                db_index=True,
                default=None,
                help_text="Start time of an actual update",
                null=True,
            ),
        ),
        migrations.RenameField(
            model_name="characterupdatestatus",
            old_name="finished_at",
            new_name="run_finished_at",
        ),
        migrations.AlterField(
            model_name="characterupdatestatus",
            name="run_finished_at",
            field=models.DateTimeField(
                db_index=True,
                default=None,
                help_text="Finish time of an update attempt",
                null=True,
            ),
        ),
        migrations.RenameField(
            model_name="characterupdatestatus",
            old_name="started_at",
            new_name="run_started_at",
        ),
        migrations.AlterField(
            model_name="characterupdatestatus",
            name="run_started_at",
            field=models.DateTimeField(
                db_index=True,
                default=None,
                help_text="Start time of an update attempt",
                null=True,
            ),
        ),
        migrations.RenameField(
            model_name="characterupdatestatus",
            old_name="last_error_message",
            new_name="error_message",
        ),
    ]
