# Generated by Django 4.0.10 on 2023-06-15 12:19

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("memberaudit", "0007_add_localization_2"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    (
                        "basic_access",
                        "Can access this app, register, and view own characters",
                    ),
                    ("share_characters", "Can share his/her characters"),
                    ("finder_access", "Can access character finder feature"),
                    ("reports_access", "Can access reports feature"),
                    ("characters_access", "Can view characters owned by others"),
                    ("exports_access", "Can access data exports"),
                    ("view_shared_characters", "Can view shared characters"),
                    ("view_same_corporation", "Can view corporation characters"),
                    ("view_same_alliance", "Can view alliance characters"),
                    ("view_everything", "Can view all characters"),
                    (
                        "notified_on_character_removal",
                        "Notified when member drops character",
                    ),
                    ("view_skill_sets", "Can view skill sets for a character"),
                ),
            },
        ),
        migrations.AddField(
            model_name="skillset",
            name="last_modified_at",
            field=models.DateTimeField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="skillset",
            name="last_modified_by",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AddField(
            model_name="skillsetgroup",
            name="last_modified_at",
            field=models.DateTimeField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="skillsetgroup",
            name="last_modified_by",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]
