# Generated by Django 4.0.10 on 2023-10-08 18:12

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("memberaudit", "0012_add_corporation_roles"),
    ]

    operations = [
        migrations.AlterField(
            model_name="characterrole",
            name="character",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="roles",
                to="memberaudit.character",
            ),
        ),
        migrations.AlterField(
            model_name="characterupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("assets", "assets"),
                    ("attributes", "attributes"),
                    ("character_details", "character details"),
                    ("contacts", "contacts"),
                    ("contracts", "contracts"),
                    ("corporation_history", "corporation history"),
                    ("fw_stats", "faction warfare statistics"),
                    ("implants", "implants"),
                    ("jump_clones", "jump clones"),
                    ("location", "location"),
                    ("loyalty", "loyalty"),
                    ("mails", "mails"),
                    ("mining_ledger", "mining ledger"),
                    ("online_status", "online status"),
                    ("planets", "planets"),
                    ("roles", "roles"),
                    ("ship", "ship"),
                    ("skills", "skills"),
                    ("skill_queue", "skill queue"),
                    ("skill_sets", "skill sets"),
                    ("standings", "standings"),
                    ("titles", "titles"),
                    ("wallet_balance", "wallet balance"),
                    ("wallet_journal", "wallet journal"),
                    ("wallet_transactions", "wallet transactions"),
                ],
                db_index=True,
                max_length=64,
            ),
        ),
        migrations.CreateModel(
            name="CharacterTitle",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title_id", models.PositiveIntegerField()),
                ("name", models.CharField(max_length=100)),
                (
                    "character",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="titles",
                        to="memberaudit.character",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.AddConstraint(
            model_name="charactertitle",
            constraint=models.UniqueConstraint(
                fields=("character", "title_id"), name="functional_pk_charactertitle"
            ),
        ),
    ]
